(function() {
  var BaseQuery, dialects, fluid, toRelation;
  var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };
  fluid = require('./fluid');
  dialects = require('./dialects');
  toRelation = require('./nodes').toRelation;
  module.exports = BaseQuery = (function() {
    function BaseQuery(rootNodeType, opts) {
      var checks, condition, description, dialectType, method, orig, table, _fn, _ref;
      if (opts == null) {
        opts = {};
      }
      this.doEcho = false;
      dialectType = opts.dialect || dialects["default"];
      this.dialect = new dialectType;
      if ((table = opts.table) != null) {
        table = toRelation(table);
      }
      this.q = new rootNodeType(table);
      if (this.dialect.pre != null) {
        _ref = this.dialect.pre;
        for (method in _ref) {
          checks = _ref[method];
          _fn = __bind(function(method, description, orig, condition) {
            return this[method] = function() {
              if (condition.apply(this.q, arguments)) {
                return orig.apply(this, arguments);
              } else {
                throw new Error(description);
              }
            };
          }, this);
          for (description in checks) {
            condition = checks[description];
            if (!(orig = this[method])) {
              continue;
            }
            _fn(method, description, orig, condition);
          }
        }
      }
    }
    BaseQuery.prototype.copy = function() {
      var c;
      c = new this.constructor(function() {});
      c.dialect = new this.dialect.constructor;
      c.q = this.q.copy();
      return c;
    };
    BaseQuery.prototype.visit = fluid(function(fn) {
      if (fn != null) {
        return fn.call(this, this);
      }
    });
    BaseQuery.prototype.echo = fluid(function() {
      return this.doEcho = true;
    });
    BaseQuery.prototype.toSql = function() {
      var sql;
      sql = this.dialect.render(this.q);
      if (this.doEcho) {
        console.log(sql);
      }
      if (this.doEcho) {
        console.dir(this.q.params());
      }
      return sql;
    };
    BaseQuery.prototype.params = function() {
      return this.q.params();
    };
    BaseQuery.prototype.execute = function(conn, cb) {
      if (typeof conn.acquire === 'function') {
        return conn.acquire(__bind(function(c) {
          return this.execute(c, function(err, res) {
            conn.release(c);
            if (err != null) {
              return cb(err);
            }
            return cb(null, res);
          });
        }, this));
      } else {
        return conn.query(this.toSql(), this.params(), cb);
      }
    };
    BaseQuery.prototype.toString = function() {
      return '[' + this.q.constructor.name + ' "' + this.toSql().substring(0, 20) + '"]';
    };
    return BaseQuery;
  })();
}).call(this);
