(function() {
  var BaseDialect, prefixIfNotEmpty;
  var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };
  prefixIfNotEmpty = function(prefix) {
    return function(node) {
      var children;
      children = this.renderNodeSet(node);
      if (children) {
        return prefix + children;
      } else {
        return '';
      }
    };
  };
  exports.BaseDialect = exports["default"] = BaseDialect = (function() {
    function BaseDialect() {}
    BaseDialect.prototype.render = function(node) {
      var name, type;
      type = node.__proto__;
      name = type.constructor.name;
      while (!this['render' + name]) {
        if ('Object' === name || !type) {
          throw new Error("Couldn't find a renderer for " + node.constructor.name);
        }
        type = type.__proto__;
        name = type.constructor.name;
      }
      return this['render' + name](node);
    };
    BaseDialect.prototype.renderString = function(s) {
      return s;
    };
    BaseDialect.prototype.renderNodeSet = function(set) {
      return set.nodes.map(__bind(function(n) {
        return this.render(n);
      }, this)).filter(function(n) {
        return n;
      }).join(set.glue);
    };
    BaseDialect.prototype.renderParenthesizedNodeSet = function(set) {
      return "(" + this.renderNodeSet(set) + ")";
    };
    BaseDialect.prototype.renderAlias = function(node) {
      return "" + (this.maybeParens(this.render(node.obj))) + " AS " + (this.render(node.alias));
    };
    BaseDialect.prototype.maybeParens = function(it) {
      if (/\s/.exec(it)) {
        return "(" + it + ")";
      } else {
        return it;
      }
    };
    BaseDialect.prototype.renderSelectProjectionSet = function(set) {
      if (!set.nodes.length) {
        return '*';
      } else {
        return this.renderNodeSet(set);
      }
    };
    BaseDialect.prototype.renderProjection = function(node) {
      return "" + (this.quote(node.source.ref())) + "." + (this.quote(node.field));
    };
    BaseDialect.prototype.quote = function(part) {
      if (/\./.exec(part)) {
        return "`" + part + "`";
      } else {
        return part;
      }
    };
    BaseDialect.prototype.renderSqlFunction = function(node) {
      return "" + (this.render(node.name)) + (this.render(node.arglist));
    };
    BaseDialect.prototype.renderValueNode = function(node) {
      return node.value;
    };
    BaseDialect.prototype.renderParameter = function(node) {
      return '?';
    };
    BaseDialect.prototype.renderSelect = prefixIfNotEmpty('SELECT ');
    BaseDialect.prototype.renderUpdate = prefixIfNotEmpty('UPDATE ');
    BaseDialect.prototype.renderInsert = prefixIfNotEmpty('INSERT INTO ');
    BaseDialect.prototype.renderInsertData = prefixIfNotEmpty('VALUES ');
    BaseDialect.prototype.renderDelete = prefixIfNotEmpty('DELETE ');
    BaseDialect.prototype.renderUpdateSet = prefixIfNotEmpty('SET ');
    BaseDialect.prototype.renderRelationSet = prefixIfNotEmpty('FROM ');
    BaseDialect.prototype.renderWhere = prefixIfNotEmpty('WHERE ');
    BaseDialect.prototype.renderGroupBy = prefixIfNotEmpty('GROUP BY ');
    BaseDialect.prototype.renderOrderBySet = prefixIfNotEmpty('ORDER BY ');
    BaseDialect.prototype.renderReturning = prefixIfNotEmpty('RETURNING ');
    BaseDialect.prototype.renderLimit = function(node) {
      if (node.value) {
        return "LIMIT " + node.value;
      } else {
        return "";
      }
    };
    BaseDialect.prototype.renderBinary = function(node) {
      return this.render(node.left) + ' ' + this.renderOp(node.op) + ' ' + this.render(node.right);
    };
    BaseDialect.prototype.renderOp = function(op) {
      switch (op.toLowerCase()) {
        case 'ne':
        case '!=':
        case '<>':
          return '!=';
        case 'eq':
        case '=':
          return '=';
        case 'lt':
        case '<':
          return '<';
        case 'gt':
        case '>':
          return '>';
        case 'lte':
        case '<=':
          return '<=';
        case 'gte':
        case '>=':
          return '>=';
        case 'like':
          return 'LIKE';
        case 'in':
          return 'IN';
        case 'is':
          return 'IS';
        default:
          throw new Error("Unsupported comparison operator: " + op);
      }
    };
    return BaseDialect;
  })();
}).call(this);
